@@ -1,6 +1,39 @@
-/*
- *  This file is part of the MR utility library.
+/** \file ducc0/infra/threading.cc
  *
+ *  \copyright Copyright (C) 2019-2023 Peter Bell, Max-Planck-Society
+ *  \authors Peter Bell, Martin Reinecke
+ */
+
+/* SPDX-License-Identifier: BSD-3-Clause OR GPL-2.0-or-later */
+
+/*
+All rights reserved.
+
+Redistribution and use in source and binary forms, with or without modification,
+are permitted provided that the following conditions are met:
+
+* Redistributions of source code must retain the above copyright notice, this
+  list of conditions and the following disclaimer.
+* Redistributions in binary form must reproduce the above copyright notice, this
+  list of conditions and the following disclaimer in the documentation and/or
+  other materials provided with the distribution.
+* Neither the name of the copyright holder nor the names of its contributors may
+  be used to endorse or promote products derived from this software without
+  specific prior written permission.
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
+ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
+ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+*/
+
+/*
  *  This code is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -16,56 +49,40 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/** \file ducc0/infra/threading.cc
- *
- *  \copyright Copyright (C) 2019-2021 Peter Bell, Max-Planck-Society
- *  \authors Peter Bell, Martin Reinecke
- */
-
 #include "ducc0/infra/threading.h"
+#include "ducc0/infra/error_handling.h"
+#include "ducc0/infra/misc_utils.h"
+#include <atomic>
+#include <utility>
 
-#ifndef DUCC0_NO_THREADING
+#ifdef DUCC0_STDCXX_LOWLEVEL_THREADING
 #include <algorithm>
 #include <stdexcept>
-#include <utility>
 #include <cstdlib>
-#include <mutex>
-#include <condition_variable>
 #include <thread>
 #include <queue>
-#include <atomic>
 #include <vector>
 #include <exception>
+#include <errno.h>
+#include <string.h>
 #if __has_include(<pthread.h>)
 #include <pthread.h>
+#if __has_include(<pthread.h>) && defined(__linux__) && defined(_GNU_SOURCE)
+#include <unistd.h>
+#endif
 #endif
-#include "ducc0/infra/misc_utils.h"
 #endif
 
 namespace ducc0 {
 
 namespace detail_threading {
 
-#ifndef DUCC0_NO_THREADING
-
-static const size_t max_threads_ = std::max<size_t>(1, std::thread::hardware_concurrency());
-
-std::atomic<size_t> default_nthreads_(max_threads_);
-
-size_t get_default_nthreads()
-  { return default_nthreads_; }
-
-void set_default_nthreads(size_t new_default_nthreads)
-  { default_nthreads_ = std::max<size_t>(1, new_default_nthreads); }
-
-size_t max_threads() { return max_threads_; }
-
 class latch
   {
     std::atomic<size_t> num_left_;
-    std::mutex mut_;
-    std::condition_variable completed_;
-    using lock_t = std::unique_lock<std::mutex>;
+    Mutex mut_;
+    CondVar completed_;
+    using lock_t = UniqueLock;
 
   public:
     latch(size_t n): num_left_(n) {}
@@ -86,12 +103,76 @@ class latch
     bool is_ready() { return num_left_ == 0; }
   };
 
+#ifdef DUCC0_STDCXX_LOWLEVEL_THREADING
+
+static long mystrtol(const char *inp)
+  {
+  auto errno_bak = errno;
+  errno=0;
+  auto res = strtol(inp, nullptr, 10);
+  MR_assert(!errno, "error during strtol conversion ", strerror(errno));
+  errno=errno_bak;
+  return res;
+  }
+
+size_t ducc0_max_threads()
+  {
+  static const size_t max_threads_ = []()
+    {
+#if __has_include(<pthread.h>) && defined(__linux__) && defined(_GNU_SOURCE)
+    cpu_set_t cpuset;
+    CPU_ZERO(&cpuset);
+    pthread_getaffinity_np(pthread_self(), sizeof(cpuset), &cpuset);
+    size_t res=0;
+    for (size_t i=0; i<CPU_SETSIZE; ++i)
+      if (CPU_ISSET(i, &cpuset)) ++res;
+#else
+    size_t res = std::max<size_t>(1, std::thread::hardware_concurrency());
+#endif
+    auto evar=getenv("DUCC0_NUM_THREADS");
+    if (!evar)
+      return res;
+    auto res2 = mystrtol(evar);
+    MR_assert(res2>=0, "invalid value in DUCC0_NUM_THREADS");
+    if (res2==0)
+      return res;
+    return std::min<size_t>(res, res2);
+    }();
+  return max_threads_;
+  }
+ 
+static thread_local bool in_parallel_region = false;
+int pin_info()
+  {
+  static const int pin_info_ = []()
+    {
+    auto evar=getenv("DUCC0_PIN_DISTANCE");
+    if (!evar)
+      return -1; // do nothing at all
+    auto res = mystrtol(evar);
+    return int(res);
+    }();
+  return pin_info_;
+  }
+int pin_offset()
+  {
+  static const int pin_offset_ = []()
+    {
+    auto evar=getenv("DUCC0_PIN_OFFSET");
+    if (!evar)
+      return 0;
+    auto res = mystrtol(evar);
+    return int(res);
+    }();
+  return pin_offset_;
+  }
+
 template <typename T> class concurrent_queue
   {
     std::queue<T> q_;
-    std::mutex mut_;
+    Mutex mut_;
     std::atomic<size_t> size_;
-    using lock_t = std::lock_guard<std::mutex>;
+    using lock_t = LockGuard;
 
   public:
     void push(T val)
@@ -117,25 +198,45 @@ template <typename T> class concurrent_queue
     bool empty() const { return size_==0; }
   };
 
-class thread_pool
+#if __has_include(<pthread.h>) && defined(__linux__) && defined(_GNU_SOURCE)
+static void do_pinning(int ithread)
+  {
+  if (pin_info()==-1) return;
+  int num_proc = sysconf(_SC_NPROCESSORS_ONLN);
+  cpu_set_t cpuset;
+  CPU_ZERO(&cpuset);
+  int cpu_wanted = pin_offset() + ithread*pin_info();
+  MR_assert((cpu_wanted>=0)&&(cpu_wanted<num_proc), "bad CPU number requested");
+  CPU_SET(cpu_wanted, &cpuset);
+  pthread_setaffinity_np(pthread_self(), sizeof(cpuset), &cpuset);
+  }
+#else
+static void do_pinning(int /*ithread*/)
+  { return; }
+#endif
+
+class ducc_thread_pool: public thread_pool
   {
   private:
     // A reasonable guess, probably close enough for most hardware
     static constexpr size_t cache_line_size = 64;
+    // align members with cache lines
     struct alignas(cache_line_size) worker
       {
       std::thread thread;
-      std::condition_variable work_ready;
-      std::mutex mut;
+      CondVar work_ready;
+      Mutex mut;
       std::atomic_flag busy_flag = ATOMIC_FLAG_INIT;
       std::function<void()> work;
 
       void worker_main(
         std::atomic<bool> &shutdown_flag,
         std::atomic<size_t> &unscheduled_tasks,
-        concurrent_queue<std::function<void()>> &overflow_work)
+        concurrent_queue<std::function<void()>> &overflow_work, size_t ithread)
         {
-        using lock_t = std::unique_lock<std::mutex>;
+        in_parallel_region = true;
+        do_pinning(ithread);
+        using lock_t = UniqueLock;
         bool expect_work = true;
         while (!shutdown_flag || expect_work)
           {
@@ -178,11 +279,11 @@ class thread_pool
       };
 
     concurrent_queue<std::function<void()>> overflow_work_;
-    std::mutex mut_;
+    Mutex mut_;
     std::vector<worker> workers_;
-    std::atomic<bool> shutdown_;
-    std::atomic<size_t> unscheduled_tasks_;
-    using lock_t = std::lock_guard<std::mutex>;
+    std::atomic<bool> shutdown_=false;
+    std::atomic<size_t> unscheduled_tasks_=0;
+    using lock_t = LockGuard;
 
     void create_threads()
       {
@@ -196,7 +297,7 @@ class thread_pool
           worker->busy_flag.clear();
           worker->work = nullptr;
           worker->thread = std::thread(
-            [worker, this]{ worker->worker_main(shutdown_, unscheduled_tasks_, overflow_work_); });
+            [worker, this, i]{ worker->worker_main(shutdown_, unscheduled_tasks_, overflow_work_, i); });
           }
         catch (...)
           {
@@ -218,14 +319,28 @@ class thread_pool
       }
 
   public:
-    explicit thread_pool(size_t nthreads):
+    explicit ducc_thread_pool(size_t nthreads):
       workers_(nthreads)
       { create_threads(); }
 
-    thread_pool(): thread_pool(max_threads_) {}
+    ducc_thread_pool(): ducc_thread_pool(ducc0_max_threads()) {}
 
-    ~thread_pool() { shutdown(); }
+    //virtual
+    ~ducc_thread_pool() { shutdown(); }
 
+    //virtual
+    size_t nthreads() const { return workers_.size(); }
+
+    //virtual
+    size_t adjust_nthreads(size_t nthreads_in) const
+      {
+      if (in_parallel_region)
+        return 1;
+      if (nthreads_in==0)
+        return ducc0_max_threads();
+      return std::min(ducc0_max_threads(), nthreads_in);
+      }
+    //virtual
     void submit(std::function<void()> work)
       {
       lock_t lock(mut_);
@@ -264,35 +379,90 @@ class thread_pool
       }
   };
 
-inline thread_pool &get_pool()
+// return a pointer to a singleton thread_pool, which is always available
+inline ducc_thread_pool *get_master_pool()
   {
-  static thread_pool pool;
+  static auto master_pool = new ducc_thread_pool(ducc0_max_threads()-1);
 #if __has_include(<pthread.h>)
   static std::once_flag f;
   call_once(f,
     []{
     pthread_atfork(
-      +[]{ get_pool().shutdown(); },  // prepare
-      +[]{ get_pool().restart(); },   // parent
-      +[]{ get_pool().restart(); }    // child
+      +[]{ get_master_pool()->shutdown(); },  // prepare
+      +[]{ get_master_pool()->restart(); },   // parent
+      +[]{ get_master_pool()->restart(); }    // child
       );
     });
 #endif
-
-  return pool;
+  return master_pool;
   }
 
+thread_local thread_pool *active_pool = get_master_pool();
+
+thread_pool *set_active_pool(thread_pool *new_pool)
+  { return std::exchange(active_pool, new_pool); }
+thread_pool *get_active_pool()
+  {
+  MR_assert(active_pool!=nullptr, "no thread pool active");
+  return active_pool;
+  }
+
+#endif
+
+#ifdef DUCC0_NO_LOWLEVEL_THREADING
+
+class ducc_pseudo_thread_pool: public thread_pool
+  {
+  public:
+    ducc_pseudo_thread_pool() {}
+
+    //virtual
+    size_t nthreads() const { return 1; }
+
+    //virtual
+    size_t adjust_nthreads(size_t /*nthreads_in*/) const
+      { return 1; }
+    //virtual
+    void submit(std::function<void()> work)
+      { work(); }
+  };
+
+// return a pointer to a singleton thread_pool, which is always available
+inline ducc_pseudo_thread_pool *get_master_pool()
+  {
+  static auto master_pool = new ducc_pseudo_thread_pool();
+  return master_pool;
+  }
+
+thread_local thread_pool *active_pool = get_master_pool();
+
+thread_pool *set_active_pool(thread_pool *new_pool)
+  { return std::exchange(active_pool, new_pool); }
+thread_pool *get_active_pool()
+  {
+  MR_assert(active_pool!=nullptr, "no thread pool active");
+  return active_pool;
+  }
+
+#endif
+
+size_t max_threads()
+  { return get_active_pool()->nthreads(); }
+size_t adjust_nthreads(size_t nthreads_in)
+  { return get_active_pool()->adjust_nthreads(nthreads_in); }
+
 class Distribution
   {
   private:
     size_t nthreads_;
-    std::mutex mut_;
+    Mutex mut_;
     size_t nwork_;
     size_t cur_;
     std::atomic<size_t> cur_dynamic_;
     size_t chunksize_;
     double fact_max_;
-    std::vector<size_t> nextstart;
+    struct alignas(64) spaced_size_t { size_t v; }; 
+    std::vector<spaced_size_t> nextstart;
     enum SchedMode { SINGLE, STATIC, DYNAMIC, GUIDED };
     SchedMode mode;
     bool single_done;
@@ -308,63 +478,59 @@ class Distribution
       single_done = false;
       nwork_ = nwork;
       nthreads_ = 1;
-      thread_map(move(f));
+      thread_map(std::move(f));
       }
     void execStatic(size_t nwork, size_t nthreads, size_t chunksize,
       std::function<void(Scheduler &)> f)
       {
       mode = STATIC;
-      nthreads_ = (nthreads==0) ? get_default_nthreads() : nthreads;
-      if (nthreads_ == 1)
-        return execSingle(nwork, move(f));
+      nthreads_ = adjust_nthreads(nthreads);
       nwork_ = nwork;
       chunksize_ = (chunksize<1) ? (nwork_+nthreads_-1)/nthreads_
                                  : chunksize;
       if (chunksize_>=nwork_)
-        return execSingle(nwork_, move(f));
+        return execSingle(nwork_, std::move(f));
+// if there are fewer chunks than threads, reduce nthreads
+      nthreads_ = std::min(nthreads_, (nwork_+chunksize_-1)/chunksize_);
       nextstart.resize(nthreads_);
       for (size_t i=0; i<nextstart.size(); ++i)
-        nextstart[i] = i*chunksize_;
-      thread_map(move(f));
+        nextstart[i].v = i*chunksize_;
+      thread_map(std::move(f));
       }
     void execDynamic(size_t nwork, size_t nthreads, size_t chunksize,
       std::function<void(Scheduler &)> f)
       {
       mode = DYNAMIC;
-      nthreads_ = (nthreads==0) ? get_default_nthreads() : nthreads;
-      if (nthreads_ == 1)
-        return execSingle(nwork, move(f));
+      nthreads_ = adjust_nthreads(nthreads);
       nwork_ = nwork;
       chunksize_ = (chunksize<1) ? 1 : chunksize;
       if (chunksize_ >= nwork)
-        return execSingle(nwork, move(f));
+        return execSingle(nwork, std::move(f));
       if (chunksize_*nthreads_>=nwork_)
-        return execStatic(nwork, nthreads, 0, move(f));
+        return execStatic(nwork, nthreads, chunksize_, std::move(f));
       cur_dynamic_ = 0;
-      thread_map(move(f));
+      thread_map(std::move(f));
       }
     void execGuided(size_t nwork, size_t nthreads, size_t chunksize_min,
       double fact_max, std::function<void(Scheduler &)> f)
       {
       mode = GUIDED;
-      nthreads_ = (nthreads==0) ? get_default_nthreads() : nthreads;
-      if (nthreads_ == 1)
-        return execSingle(nwork, move(f));
+      nthreads_ = adjust_nthreads(nthreads);
       nwork_ = nwork;
       chunksize_ = (chunksize_min<1) ? 1 : chunksize_min;
       if (chunksize_*nthreads_>=nwork_)
-        return execStatic(nwork, nthreads, 0, move(f));
+        return execStatic(nwork, nthreads, chunksize_, std::move(f));
       fact_max_ = fact_max;
       cur_ = 0;
-      thread_map(move(f));
+      thread_map(std::move(f));
       }
     void execParallel(size_t nthreads, std::function<void(Scheduler &)> f)
       {
       mode = STATIC;
-      nthreads_ = (nthreads==0) ? get_default_nthreads() : nthreads;
+      nthreads_ = adjust_nthreads(nthreads);
       nwork_ = nthreads_;
       chunksize_ = 1;
-      thread_map(move(f));
+      thread_map(std::move(f));
       }
     Range getNext(size_t thread_id)
       {
@@ -378,10 +544,10 @@ class Distribution
           }
         case STATIC:
           {
-          if (nextstart[thread_id]>=nwork_) return Range();
-          size_t lo=nextstart[thread_id];
+          if (nextstart[thread_id].v>=nwork_) return Range();
+          size_t lo=nextstart[thread_id].v;
           size_t hi=std::min(lo+chunksize_,nwork_);
-          nextstart[thread_id] += nthreads_*chunksize_;
+          nextstart[thread_id].v += nthreads_*chunksize_;
           return Range(lo, hi);
           }
         case DYNAMIC:
@@ -392,7 +558,7 @@ class Distribution
           }
         case GUIDED:
           {
-          std::unique_lock<std::mutex> lck(mut_);
+          LockGuard lck(mut_);
           if (cur_>=nwork_) return Range();
           auto rem = nwork_-cur_;
           size_t tmp = size_t((fact_max_*double(rem))/double(nthreads_));
@@ -421,6 +587,40 @@ class MyScheduler: public Scheduler
     virtual Range getNext() { return dist_.getNext(ithread_); }
   };
 
+template<typename T> class ScopedValueChanger
+  {
+  private:
+    T &object;
+    T original_value;
+
+  public:
+    ScopedValueChanger(T &object_, T new_value)
+      : object(object_), original_value(object_) { object=new_value; }
+    ~ScopedValueChanger()
+      { object=original_value; }
+  };
+
+#define DUCC0_HIERARCHICAL_SUBMISSION
+#ifdef DUCC0_HIERARCHICAL_SUBMISSION
+
+// The next two definitions are taken from TensorFlow sources.
+// Copyright 2015 The TensorFlow Authors.
+
+// Basic y-combinator implementation.
+template <class Func> struct YCombinatorImpl {
+  Func func;
+  template <class... Args>
+  decltype(auto) operator()(Args&&... args) const {
+    return func(*this, std::forward<Args>(args)...);
+  }
+};
+
+template <class Func> YCombinatorImpl<std::decay_t<Func>> YCombinator(Func&& func) {
+  return YCombinatorImpl<std::decay_t<Func>>{std::forward<Func>(func)};
+}
+
+#endif
+
 void Distribution::thread_map(std::function<void(Scheduler &)> f)
   {
   if (nthreads_ == 1)
@@ -430,27 +630,76 @@ void Distribution::thread_map(std::function<void(Scheduler &)> f)
     return;
     }
 
-  auto & pool = get_pool();
-  latch counter(nthreads_);
   std::exception_ptr ex;
-  std::mutex ex_mut;
-  for (size_t i=0; i<nthreads_; ++i)
+  Mutex ex_mut;
+  // we "copy" the currently active thread pool to all executing threads
+  // during the execution of f. This ensures that possible nested parallel
+  // regions are handled by the same pool and not by the one that happens
+  // to be active on the worker threads.
+  // Alternatively we could put a "no-threading" thread pool onto the executing
+  // threads, which executes everything sequentially on its own thread,
+  // automatically prohibiting nested parallelism.
+  auto pool = get_active_pool();
+
+#ifdef DUCC0_HIERARCHICAL_SUBMISSION
+
+  latch counter(nthreads_);
+  // distribute work to helper threads, in a recursive fashion
+  auto new_f = YCombinator([this, &f, &counter, &ex, &ex_mut, pool](auto &new_f, size_t istart, size_t step) -> void {
+    try
+      {
+      ScopedValueChanger changer(in_parallel_region, true);
+      ScopedUseThreadPool guard(*pool);
+      for(; step>0; step>>=1)
+        if(istart+step<nthreads_)
+          pool->submit([this, &f, &new_f, &counter, &ex, &ex_mut, pool, istart, step]()
+            {new_f(istart+step, step>>1);});
+      MyScheduler sched(*this, istart);
+      f(sched);
+      }
+    catch (...)
+      {
+      LockGuard lock(ex_mut);
+      ex = std::current_exception();
+      }
+    counter.count_down();
+    });
+
+  size_t biggest_step=1;
+  while (biggest_step*2<nthreads_) biggest_step<<=1;
+  new_f(0, biggest_step);
+
+#else  // sequential submission
+
+  latch counter(nthreads_-1);
+  for (size_t i=1; i<nthreads_; ++i)
     {
-    pool.submit(
-      [this, &f, i, &counter, &ex, &ex_mut] {
+    pool->submit(
+      [this, &f, i, &counter, &ex, &ex_mut, pool] {
       try
         {
+        ScopedUseThreadPool guard(*pool);
         MyScheduler sched(*this, i);
         f(sched);
         }
       catch (...)
         {
-        std::lock_guard<std::mutex> lock(ex_mut);
+        LockGuard lock(ex_mut);
         ex = std::current_exception();
         }
       counter.count_down();
       });
     }
+  {
+  // do remaining work directly on this thread
+  ScopedValueChanger changer(in_parallel_region, true);
+  MyScheduler sched(*this, 0);
+  f(sched);
+  }
+
+#endif
+#undef DUCC0_HIERARCHICAL_SUBMISSION
+
   counter.wait();
   if (ex)
     std::rethrow_exception(ex);
@@ -459,35 +708,41 @@ void Distribution::thread_map(std::function<void(Scheduler &)> f)
 void execSingle(size_t nwork, std::function<void(Scheduler &)> func)
   {
   Distribution dist;
-  dist.execSingle(nwork, move(func));
+  dist.execSingle(nwork, std::move(func));
   }
 void execStatic(size_t nwork, size_t nthreads, size_t chunksize,
   std::function<void(Scheduler &)> func)
   {
   Distribution dist;
-  dist.execStatic(nwork, nthreads, chunksize, move(func));
+  dist.execStatic(nwork, nthreads, chunksize, std::move(func));
   }
 void execDynamic(size_t nwork, size_t nthreads, size_t chunksize,
   std::function<void(Scheduler &)> func)
   {
   Distribution dist;
-  dist.execDynamic(nwork, nthreads, chunksize, move(func));
+  dist.execDynamic(nwork, nthreads, chunksize, std::move(func));
   }
 void execGuided(size_t nwork, size_t nthreads, size_t chunksize_min,
   double fact_max, std::function<void(Scheduler &)> func)
   {
   Distribution dist;
-  dist.execGuided(nwork, nthreads, chunksize_min, fact_max, move(func));
+  dist.execGuided(nwork, nthreads, chunksize_min, fact_max, std::move(func));
   }
 void execParallel(size_t nthreads, std::function<void(Scheduler &)> func)
   {
   Distribution dist;
-  dist.execParallel(nthreads, move(func));
+  dist.execParallel(nthreads, std::move(func));
+  }
+void execParallel(size_t nthreads, std::function<void(size_t)> func)
+  {
+  Distribution dist;
+  dist.execParallel(nthreads, [&](Scheduler &sched)
+    { func(sched.thread_num()); });
   }
 void execParallel(size_t work_lo, size_t work_hi, size_t nthreads,
   std::function<void(size_t, size_t)> func)
   {
-  nthreads = (nthreads==0) ? get_default_nthreads() : nthreads;
+  nthreads = adjust_nthreads(nthreads);
   execParallel(nthreads, [&](Scheduler &sched)
     {
     auto tid = sched.thread_num();
@@ -498,7 +753,7 @@ void execParallel(size_t work_lo, size_t work_hi, size_t nthreads,
 void execParallel(size_t work_lo, size_t work_hi, size_t nthreads,
   std::function<void(size_t, size_t, size_t)> func)
   {
-  nthreads = (nthreads==0) ? get_default_nthreads() : nthreads;
+  nthreads = adjust_nthreads(nthreads);
   execParallel(nthreads, [&](Scheduler &sched)
     {
     auto tid = sched.thread_num();
@@ -507,64 +762,4 @@ void execParallel(size_t work_lo, size_t work_hi, size_t nthreads,
     });
   }
 
-#else
-
-size_t get_default_nthreads() { return 1; }
-void set_default_nthreads(size_t /* new_default_nthreads */) {}
-size_t max_threads() { return 1; }
-
-class MyScheduler: public Scheduler
-  {
-  private:
-    size_t nwork_;
-
-  public:
-    MyScheduler(size_t nwork) : nwork_(nwork) {}
-    virtual size_t num_threads() const { return 1; }
-    virtual size_t thread_num() const { return 0; }
-    virtual Range getNext()
-      {
-      Range res(0, nwork_);
-      nwork_=0;
-      return res;
-      }
-  };
-
-void execSingle(size_t nwork, std::function<void(Scheduler &)> func)
-  {
-  MyScheduler sched(nwork);
-  func(sched);
-  }
-void execStatic(size_t nwork, size_t, size_t,
-  std::function<void(Scheduler &)> func)
-  {
-  MyScheduler sched(nwork);
-  func(sched);
-  }
-void execDynamic(size_t nwork, size_t, size_t,
-  std::function<void(Scheduler &)> func)
-  {
-  MyScheduler sched(nwork);
-  func(sched);
-  }
-void execGuided(size_t nwork, size_t, size_t, double,
-  std::function<void(Scheduler &)> func)
-  {
-  MyScheduler sched(nwork);
-  func(sched);
-  }
-void execParallel(size_t, std::function<void(Scheduler &)> func)
-  {
-  MyScheduler sched(1);
-  func(sched);
-  }
-void execParallel(size_t work_lo, size_t work_hi, size_t,
-  std::function<void(size_t, size_t)> func)
-  { func(lo, hi); }
-void execParallel(size_t work_lo, size_t work_hi, size_t,
-  std::function<void(size_t, size_t, size_t)> func)
-  { func(0, lo, hi); }
-
-#endif
-
 }}
