@@ -461,16 +461,16 @@ class WStackingGridder {
    * Get the kernel function and its interpolation method used for gridding.
    * @returns The currently selected gridding mode.
    */
-  GridMode GetGridMode() const { return _gridMode; }
+  GriddingKernelMode GetGridMode() const { return _gridMode; }
 
   /**
    * Set the kernel function and its interpolation method used for gridding.
    * @param mode The new gridding mode.
    */
-  void SetGridMode(GridMode mode) {
+  void SetGridMode(GriddingKernelMode mode) {
     if (mode != _gridMode) {
       _gridMode = mode;
-      if (_gridMode != GridMode::NearestNeighbourGridding) makeKernels();
+      if (_gridMode != GriddingKernelMode::NearestNeighbour) makeKernels();
     }
   }
 
@@ -503,8 +503,8 @@ class WStackingGridder {
    * @param dm shift of image centre in 'm' (North) direction
    */
   void SetDenormalPhaseCentre(double dl, double dm) {
-    _phaseCentreDL = dl;
-    _phaseCentreDM = dm;
+    _l_shift = dl;
+    _m_shift = dm;
   }
 
   /**
@@ -527,8 +527,8 @@ class WStackingGridder {
    * @param oversampling Oversampling factor of kernel
    * @param size UV-cell size of the kernel
    */
-  static void GetKernel(GridMode gridMode, double *kernel, size_t oversampling,
-                        size_t size);
+  static void GetKernel(GriddingKernelMode gridMode, double *kernel,
+                        size_t oversampling, size_t size);
 
   /**
    * Get width of image as specified during construction. This is the full
@@ -617,13 +617,13 @@ class WStackingGridder {
   const size_t _width, _height;
   const double _pixelSizeX, _pixelSizeY;
   size_t _nWLayers, _nPasses, _curLayerRangeIndex;
-  double _minW, _maxW, _phaseCentreDL, _phaseCentreDM;
+  double _minW, _maxW, _l_shift, _m_shift;
   bool _isComplex, _imageConjugatePart;
 #ifndef AVOID_CASACORE
   aocommon::BandData _bandData;
 #endif
 
-  GridMode _gridMode;
+  GriddingKernelMode _gridMode;
   size_t _overSamplingFactor, _kernelSize;
   std::vector<double> _1dKernel;
   std::vector<std::vector<num_t>> _griddingKernels;
