@@ -1,13 +1,12 @@
 #ifndef BINNED_UV_OUTPUT_H
 #define BINNED_UV_OUTPUT_H
 
-#include "imagebufferallocator.h"
-
-#include "../rmsimage.h"
+#include <schaapcommon/fft/resampler.h>
 
 #include <aocommon/image.h>
 #include <aocommon/fits/fitsreader.h>
 #include <aocommon/fits/fitswriter.h>
+#include <aocommon/logger.h>
 
 #include <schaapcommon/fft/resampler.h>
 
@@ -15,26 +14,26 @@
 
 class BinnedUVOutput {
  public:
-  static void Make(const std::string& uvCoveragePrefix,
-                   const std::string& dirtyPrefix, double psfLimit = 1e-4) {
-    ImageBufferAllocator allocator;
-
+  static void Make(const std::string &uvCoveragePrefix,
+                   const std::string &dirtyPrefix, double psfLimit = 1e-4) {
     aocommon::FitsReader dirtyReader(dirtyPrefix + "-dirty.fits");
     aocommon::FitsReader psfReader(dirtyPrefix + "-psf.fits");
     aocommon::FitsReader uvCoverageReader(uvCoveragePrefix + "-psf.fits");
-    const size_t width = dirtyReader.ImageWidth(),
-                 height = dirtyReader.ImageHeight();
-    aocommon::Image dirty(width, height, allocator),
-        psf(width, height, allocator), uvCovPsf(width, height, allocator),
-        realUV(width, height, allocator), imagUV(width, height, allocator),
-        realPsfUV(width, height, allocator),
-        imagPsfUV(width, height, allocator),
-        realPsfUVCoverage(width, height, allocator),
-        imagPsfUVCoverage(width, height, allocator),
-        binned(width, height, allocator);
-    dirtyReader.Read(dirty.data());
-    psfReader.Read(psf.data());
-    uvCoverageReader.Read(uvCovPsf.data());
+    const size_t width = dirtyReader.ImageWidth();
+    const size_t height = dirtyReader.ImageHeight();
+    aocommon::Image dirty(width, height);
+    aocommon::Image psf(width, height);
+    aocommon::Image uvCovPsf(width, height);
+    aocommon::Image realUV(width, height);
+    aocommon::Image imagUV(width, height);
+    aocommon::Image realPsfUV(width, height);
+    aocommon::Image imagPsfUV(width, height);
+    aocommon::Image realPsfUVCoverage(width, height);
+    aocommon::Image imagPsfUVCoverage(width, height);
+    aocommon::Image binned(width, height);
+    dirtyReader.Read(dirty.Data());
+    psfReader.Read(psf.Data());
+    uvCoverageReader.Read(uvCovPsf.Data());
 
     double nVis, normF, vwSum;
     if (!dirtyReader.ReadDoubleKeyIfExists("WSCNVIS", nVis))
@@ -57,12 +56,12 @@ class BinnedUVOutput {
     dirty *= normF * nVis / (2.0 * sqrt(width * height) * vwSum);
     psf *= normF * nVis / (2.0 * sqrt(width * height) * vwSum);
     uvCovPsf *= normFUVC * nVisUVC / (2.0 * sqrt(width * height) * vwSumUVC);
-    schaapcommon::fft::Resampler fft(width, height, width, height, 1, false);
-    fft.SingleFT(dirty.data(), realUV.data(), imagUV.data());
-    fft.SingleFT(psf.data(), realPsfUV.data(), imagPsfUV.data());
-    fft.SingleFT(uvCovPsf.data(), realPsfUVCoverage.data(),
-                 imagPsfUVCoverage.data());
-    for (size_t i = 0; i != dirty.size(); ++i) {
+    schaapcommon::fft::Resampler fft(width, height, width, height, 1);
+    fft.SingleFT(dirty.Data(), realUV.Data(), imagUV.Data());
+    fft.SingleFT(psf.Data(), realPsfUV.Data(), imagPsfUV.Data());
+    fft.SingleFT(uvCovPsf.Data(), realPsfUVCoverage.Data(),
+                 imagPsfUVCoverage.Data());
+    for (size_t i = 0; i != dirty.Size(); ++i) {
       std::complex<double> psfVal =
           std::complex<double>(realPsfUV[i], imagPsfUV[i]);
       std::complex<double> newVal =
@@ -80,12 +79,13 @@ class BinnedUVOutput {
     size_t nBins = 0;
     double effVisSum = 0.0;
     for (size_t y = 0; y != height; ++y) {
-      const double *realPsfPtr = &realPsfUV[y * width],
-                   *imagPsfPtr = &imagPsfUV[y * width],
-                   *realPsfUVCovPtr = &realPsfUVCoverage[y * width],
-                   *imagPsfUVCovPtr = &imagPsfUVCoverage[y * width];
-      double *realPtr = &realUV[y * width], *imagPtr = &imagUV[y * width],
-             *binnedPtr = &binned[y * width];
+      const float *realPsfPtr = &realPsfUV[y * width];
+      const float *imagPsfPtr = &imagPsfUV[y * width];
+      const float *realPsfUVCovPtr = &realPsfUVCoverage[y * width];
+      const float *imagPsfUVCovPtr = &imagPsfUVCoverage[y * width];
+      float *realPtr = &realUV[y * width];
+      float *imagPtr = &imagUV[y * width];
+      float *binnedPtr = &binned[y * width];
       double vInLambda =
           double(int(height) / 2 - int(y)) / (pixelSizeY * height);
       for (size_t x = 0; x != width; ++x) {
@@ -111,10 +111,10 @@ class BinnedUVOutput {
       }
     }
     aocommon::FitsWriter writer(dirtyReader);
-    writer.Write(dirtyPrefix + "-binneduvoutput.fits", binned.data());
-    Logger::Info << "UV bins were written. " << nBins
-                 << " bins were selected, with an effective nvis of "
-                 << effVisSum << " (actual nvis=" << nVis << ").\n";
+    writer.Write(dirtyPrefix + "-binneduvoutput.fits", binned.Data());
+    aocommon::Logger::Info << "UV bins were written. " << nBins
+                           << " bins were selected, with an effective nvis of "
+                           << effVisSum << " (actual nvis=" << nVis << ").\n";
   }
 };
 
