@@ -1,12 +1,14 @@
 #include "commandline.h"
 #include "wsclean.h"
 
-#include "../io/logger.h"
-#include "../system/check_openblas_multithreading.h"
+#include <aocommon/checkblas.h>
+#include <aocommon/logger.h>
 
 #include <exception>
 #include <iostream>
 
+using aocommon::Logger;
+
 int main(int argc, char* argv[]) {
   try {
     check_openblas_multithreading();
