@@ -1,11 +1,21 @@
 #include "averagingmsrowprovider.h"
 
-#include "../structures/multibanddata.h"
-
-#include "../io/logger.h"
+#include <aocommon/logger.h>
+#include <aocommon/multibanddata.h>
 
 #include <casacore/tables/Tables/ArrayColumn.h>
 
+using aocommon::Logger;
+
+namespace {
+template <typename ArrayT>
+void copyAndResize(const ArrayT& source, ArrayT& destination) {
+  destination.resize(source.shape());
+  const size_t bufferSize = source.shape()[0] * source.shape()[1];
+  std::copy_n(source.data(), bufferSize, destination.data());
+}
+}  // namespace
+
 AveragingMSRowProvider::AveragingMSRowProvider(
     double nWavelengthsAveraging, const string& msPath,
     const MSSelection& selection,
@@ -14,7 +24,7 @@ AveragingMSRowProvider::AveragingMSRowProvider(
     : MSRowProvider(msPath, selection, selectedDataDescIds, dataColumnName,
                     requireModel),
       _fieldId(fieldId) {
-  casacore::MSAntenna antennaTable(_ms.antenna());
+  casacore::MSAntenna antennaTable(Ms().antenna());
   _nAntennae = antennaTable.nrow();
 
   casacore::ArrayColumn<double> positionColumn(
@@ -32,15 +42,14 @@ AveragingMSRowProvider::AveragingMSRowProvider(
   _nElements = selectedDataDescIds.size() * _nAntennae * _nAntennae;
   _averagingFactors.assign(_nElements, 0.0);
   _buffers.resize(_nElements);
-  MultiBandData bands(_ms.spectralWindow(), _ms.dataDescription());
+  aocommon::MultiBandData bands(Ms().spectralWindow(), Ms().dataDescription());
 
   double dt = (EndTime() - StartTime()) / (EndTimestep() - StartTimestep());
   Logger::Debug << "Assuming integration time of " << dt * (24.0 * 60.0 * 60.0)
                 << " seconds.\n";
 
   size_t element = 0;
-  size_t averagingSum = 0, minAvgFactor = std::numeric_limits<size_t>::max(),
-         maxAvgFactor = 0;
+  size_t minAvgFactor = std::numeric_limits<size_t>::max(), maxAvgFactor = 0;
   for (size_t a1 = 0; a1 != _nAntennae; ++a1) {
     Pos pos1 = positions[a1];
     for (size_t a2 = 0; a2 != _nAntennae; ++a2) {
@@ -52,13 +61,12 @@ AveragingMSRowProvider::AveragingMSRowProvider(
       for (std::map<size_t, size_t>::const_iterator spwIter =
                selectedDataDescIds.begin();
            spwIter != selectedDataDescIds.end(); ++spwIter) {
-        BandData band = bands[spwIter->first];
+        aocommon::BandData band = bands[spwIter->first];
         double lambda = band.SmallestWavelength();
         double nWavelengthsPerIntegration = 2.0 * M_PI * dist / lambda * dt;
         _averagingFactors[element] = std::max<size_t>(
             size_t(floor(nWavelengthsAveraging / nWavelengthsPerIntegration)),
             1);
-        averagingSum += _averagingFactors[element];
         if (a1 != a2) {
           minAvgFactor =
               std::min<size_t>(minAvgFactor, _averagingFactors[element]);
@@ -83,10 +91,6 @@ AveragingMSRowProvider::AveragingMSRowProvider(
   _rowCount = 0;
   _averagedRowCount = 0;
 
-  _currentData = DataArray(DataShape());
-  _currentModel = DataArray(DataShape());
-  _currentFlags = FlagArray(DataShape());
-  _currentWeights = WeightArray(DataShape());
   _averagedDataDescId = _currentDataDescId;
   _flushPosition = 0;
 
@@ -97,28 +101,30 @@ AveragingMSRowProvider::AveragingMSRowProvider(
 }
 
 bool AveragingMSRowProvider::processCurrentTimestep() {
-  size_t a1 = _antenna1Column(_currentRow);
-  size_t a2 = _antenna2Column(_currentRow);
+  MsColumns& columns = Columns();
+  const size_t a1 = columns.antenna_1(_currentRow);
+  const size_t a2 = columns.antenna_2(_currentRow);
   _averagedDataDescId = _currentDataDescId;
   _averagedAntenna1Index = a1;
   _averagedAntenna2Index = a2;
 
-  size_t spwCount = selectedDataDescIds().size();
-  size_t elementIndex = spwCount * (a2 + a1 * _nAntennae) +
-                        selectedDataDescIds().find(_averagedDataDescId)->second;
-  size_t avgFactor = _averagingFactors[elementIndex];
+  const size_t spwCount = selectedDataDescIds().size();
+  const size_t elementIndex =
+      spwCount * (a2 + a1 * _nAntennae) +
+      selectedDataDescIds().find(_averagedDataDescId)->second;
+  const size_t avgFactor = _averagingFactors[elementIndex];
   _averageFactorSum += avgFactor;
   ++_rowCount;
 
-  _dataColumn.get(_currentRow, _currentData);
-  _flagColumn.get(_currentRow, _currentFlags);
-  getCurrentWeights(_currentWeights);
-  if (requireModel()) _modelColumn->get(_currentRow, _currentModel);
+  columns.data.get(_currentRow, _currentData, true);
+  columns.flag.get(_currentRow, _currentFlags, true);
+  getCurrentWeights(_currentWeights, _currentData.shape());
+  if (requireModel()) _modelColumn->get(_currentRow, _currentModel, true);
 
   if (avgFactor == 1)
     return true;
   else {
-    size_t bufferSize = DataShape()[0] * DataShape()[1];
+    const size_t bufferSize = _currentData.shape()[0] * _currentData.shape()[1];
     AveragingBuffer& buffer = _buffers[elementIndex];
     if (!buffer.IsInitialized()) buffer.Initialize(bufferSize, requireModel());
 
@@ -132,7 +138,7 @@ bool AveragingMSRowProvider::processCurrentTimestep() {
                      _currentWeights.data(), _currentUVWArray.data(),
                      _currentTime);
 
-    bool foundFullBuffer = (buffer.AveragedDataCount() == avgFactor);
+    const bool foundFullBuffer = (buffer.AveragedDataCount() == avgFactor);
     if (foundFullBuffer) {
       if (requireModel())
         buffer.Get(bufferSize, _currentData.data(), _currentModel.data(),
@@ -163,14 +169,14 @@ void AveragingMSRowProvider::NextRow() {
 
   if (MSRowProvider::AtEnd()) {
     // There might be residual data in the buffers which have to be read out
-    AveragingBuffer* buffer = 0;
+    AveragingBuffer* buffer = nullptr;
     do {
       buffer = &_buffers[_flushPosition];
       ++_flushPosition;
     } while (buffer->AveragedDataCount() == 0 && _flushPosition < _nElements);
 
-    if (buffer != 0 && buffer->AveragedDataCount() != 0) {
-      size_t bufferSize = DataShape()[0] * DataShape()[1];
+    if (buffer != nullptr && buffer->AveragedDataCount() != 0) {
+      size_t bufferSize = _currentData.shape()[0] * _currentData.shape()[1];
       if (requireModel())
         buffer->Get(bufferSize, _currentData.data(), _currentModel.data(),
                     _currentFlags.data(), _currentWeights.data(),
@@ -198,10 +204,9 @@ void AveragingMSRowProvider::ReadData(MSRowProvider::DataArray& data,
                                       uint32_t& dataDescId, uint32_t& antenna1,
                                       uint32_t& antenna2, uint32_t& fieldId,
                                       double& time) {
-  size_t bufferSize = DataShape()[0] * DataShape()[1];
-  std::copy_n(_currentData.data(), bufferSize, data.data());
-  std::copy_n(_currentFlags.data(), bufferSize, flags.data());
-  std::copy_n(_currentWeights.data(), bufferSize, weights.data());
+  copyAndResize(_currentData, data);
+  copyAndResize(_currentFlags, flags);
+  copyAndResize(_currentWeights, weights);
   u = _currentUVWArray.data()[0];
   v = _currentUVWArray.data()[1];
   w = _currentUVWArray.data()[2];
@@ -213,8 +218,7 @@ void AveragingMSRowProvider::ReadData(MSRowProvider::DataArray& data,
 }
 
 void AveragingMSRowProvider::ReadModel(MSRowProvider::DataArray& model) {
-  size_t bufferSize = DataShape()[0] * DataShape()[1];
-  std::copy_n(_currentModel.data(), bufferSize, model.data());
+  copyAndResize(_currentModel, model);
 }
 
 void AveragingMSRowProvider::OutputStatistics() const {
