@@ -12,16 +12,17 @@ void DirectMSRowProvider::ReadData(MSRowProvider::DataArray& data,
   v = _currentUVWArray(1);
   w = _currentUVWArray(2);
   dataDescId = _currentDataDescId;
-  _dataColumn.get(_currentRow, data, true);
-  _flagColumn.get(_currentRow, flags, true);
-  antenna1 = _antenna1Column(_currentRow);
-  antenna2 = _antenna2Column(_currentRow);
-  fieldId = _fieldIdColumn(_currentRow);
-  time = _timeColumn(_currentRow);
+  MsColumns& columns = Columns();
+  columns.data.get(_currentRow, data, true);
+  columns.flag.get(_currentRow, flags, true);
+  antenna1 = columns.antenna_1(_currentRow);
+  antenna2 = columns.antenna_2(_currentRow);
+  fieldId = columns.field_id(_currentRow);
+  time = columns.time(_currentRow);
 
-  getCurrentWeights(weights);
+  getCurrentWeights(weights, data.shape());
 }
 
 void DirectMSRowProvider::ReadModel(MSRowProvider::DataArray& model) {
-  _modelColumn->get(_currentRow, model);
+  _modelColumn->get(_currentRow, model, true);
 }
