@@ -46,6 +46,8 @@ class TimestepBuffer final : public MSProvider {
 
   double StartTime() override { return _msProvider->StartTime(); }
 
+  size_t DataDescId() override { return _msProvider->DataDescId(); }
+
   void MakeIdToMSRowMapping(std::vector<size_t>& idToMSRow) override {
     _msProvider->MakeIdToMSRowMapping(idToMSRow);
   }
@@ -60,6 +62,8 @@ class TimestepBuffer final : public MSProvider {
 
   size_t NPolarizations() override { return _msProvider->NPolarizations(); }
 
+  const aocommon::BandData& Band() override { return _msProvider->Band(); }
+
  private:
   struct RowData {
     std::vector<std::complex<float>> data, model;
