@@ -71,6 +71,19 @@ def makeWCS(centreX, centreY, refRA, refDec, crdelt=0.066667):
     return w
 
 
+def convert_to_deg(array_ra, array_dec):
+    try:
+        # Degree format
+        new_ra = Angle(array_ra, unit="degree")
+        new_dec = Angle(array_dec, unit="degree")
+    except ValueError:
+        # Skymodel.txt format
+        new_ra = Angle(array_ra, unit="hourangle")
+        new_dec = Angle(np.char.replace(array_dec, ".", ":", 2), unit="degree")
+
+    return [new_ra.deg, new_dec.deg]
+
+
 def generate_centroids_from_source_catalog(catalog_file, npoints, w):
     """
     Generate centroids from a source cataloge, such as gleam-osm.
@@ -90,13 +103,34 @@ def generate_centroids_from_source_catalog(catalog_file, npoints, w):
         Numpy (npoints, 2) array with pixel coordinates of . Dimension:
     """
 
-    catalog = np.genfromtxt(catalog_file, delimiter=",")
+    catalog = np.genfromtxt(
+        catalog_file, delimiter=",", dtype=str, encoding=None
+    )
     source_idx = np.argsort(catalog[:, 2])[: -npoints - 1 : -1]
-    x, y = w.wcs_world2pix(catalog[source_idx, 0], catalog[source_idx, 1], 1)
+    catalog = np.char.strip(catalog)
+
+    # Search the keywords "ra" and "dec" in the first line of the catalog
+    # to get the right indexes.
+    # If not found use the indexes 0 and 1 as in the "gleam.osm" catalog
+    try:
+        index_ra = list(np.char.lower(catalog[0, :])).index("ra")
+        index_dec = list(np.char.lower(catalog[0, :])).index("dec")
+    except:
+        index_ra = 0
+        index_dec = 1
+
+    # Convert Ra/Dec unit to degrees
+    [ra_coords, dec_coords] = convert_to_deg(
+        catalog[source_idx, index_ra], catalog[source_idx, index_dec]
+    )
+
+    x, y = w.wcs_world2pix(ra_coords, dec_coords, 1)
     return np.vstack((x.flatten(), y.flatten())).T
 
 
-def tessellate(x_pix, y_pix, w, dist_pix, bbox, nouter=64, plot_tessellation=True):
+def tessellate(
+    x_pix, y_pix, w, dist_pix, bbox, nouter=64, plot_tessellation=True
+):
     """
     Returns Voronoi tessellation vertices
 
@@ -188,7 +222,9 @@ def tessellate(x_pix, y_pix, w, dist_pix, bbox, nouter=64, plot_tessellation=Tru
     verts = [verts[i] for i in ind]
 
     ra_point, dec_point = w.wcs_pix2world(x_pix, y_pix, 1)
-    return [Polygon(vert) for vert in verts], np.vstack((ra_point, dec_point)).T
+    return [Polygon(vert) for vert in verts], np.vstack(
+        (ra_point, dec_point)
+    ).T
 
 
 def generate_centroids(
@@ -234,11 +270,15 @@ def generate_centroids(
     X, Y = np.meshgrid(x, y)
 
     xtol = np.diff(x)[0]
-    dX = np.random.uniform(low=-distort_x * xtol, high=distort_x * xtol, size=X.shape)
+    dX = np.random.uniform(
+        low=-distort_x * xtol, high=distort_x * xtol, size=X.shape
+    )
     X = X + dX
 
     ytol = np.diff(y)[0]
-    dY = np.random.uniform(low=-distort_x * ytol, high=distort_y * ytol, size=Y.shape)
+    dY = np.random.uniform(
+        low=-distort_x * ytol, high=distort_y * ytol, size=Y.shape
+    )
     Y = Y + dY
     return X.flatten(), Y.flatten()
 
@@ -285,12 +325,14 @@ def write_ds9(fname, polygons, points=None):
         for i, polygon in enumerate(polygons):
             poly_string = "polygon("
             xv, yv = polygon.exterior.xy
-            for (x, y) in zip(xv[:-1], yv[:-1]):
+            for x, y in zip(xv[:-1], yv[:-1]):
                 poly_string = f"{poly_string}{x:.5f},{y:.5f},"
             # Strip trailing comma
             poly_string = poly_string[:-1] + ")"
             if points is not None:
-                poly_string += f"\npoint({points[i, 0]:.5f}, {points[i, 1]:.5f})"
+                poly_string += (
+                    f"\npoint({points[i, 0]:.5f}, {points[i, 1]:.5f})"
+                )
             polygon_strings.append(poly_string)
         f.write("\n".join(polygon_strings))
 
@@ -337,7 +379,12 @@ def main(args):
     elif args.sourcecatalog:
         # Add two points to account for outer points that will be stripped away
         x_background, y_background = generate_centroids(
-            xmin, ymin, xmax, ymax, args.backgroundfacets + 2, args.backgroundfacets + 2
+            xmin,
+            ymin,
+            xmax,
+            ymax,
+            args.backgroundfacets + 2,
+            args.backgroundfacets + 2,
         )
         xy = generate_centroids_from_source_catalog(
             args.sourcecatalog[0], int(args.sourcecatalog[1]), w
@@ -355,7 +402,9 @@ def main(args):
     )
 
     write_ds9(
-        args.outputfile, facets, points=points if args.writevoronoipoints else None
+        args.outputfile,
+        facets,
+        points=points if args.writevoronoipoints else None,
     )
 
 
