@@ -77,6 +77,7 @@ void ImageWeights::Grid(MSProvider& msProvider,
   assert(!_isGriddingFinished);
   const size_t polarizationCount = msProvider.NPolarizations();
   if (_weightMode.RequiresGridding()) {
+    assert(selectedBand.ChannelCount() == msProvider.NChannels());
     aocommon::UVector<float> weightBuffer(selectedBand.ChannelCount() *
                                           polarizationCount);
 
@@ -124,7 +125,9 @@ void ImageWeights::FinishGridding() {
           5.0 * std::exp((-M_LN10) * _weightMode.BriggsRobustness());
       double sSq = numeratorSqrt * numeratorSqrt / avgW;
       for (double& val : _grid) {
-        val = 1.0 / (1.0 + val * sSq);
+        // Values without coverage are left to zero, because they would
+        // otherwise affect the weight rank filter.
+        if (val != 0.0) val = 1.0 / (1.0 + val * sSq);
       }
     } break;
     case WeightMode::UniformWeighted: {
