@@ -1,6 +1,7 @@
 #include "imagingtable.h"
 
 #include <aocommon/logger.h>
+#include <schaapcommon/facets/facet.h>
 
 #include <algorithm>
 #include <cassert>
@@ -11,11 +12,17 @@
 using aocommon::Logger;
 
 ImagingTable::ImagingTable(const std::vector<EntryPtr>& entries)
-    : _entries(entries),
-      _independentGroups(),
-      _facetGroups(),
-      _facets(),
-      _squaredGroups() {
+    : _entries(entries), _independentGroups(), _facets(), _squaredGroups() {
+  Update();
+}
+
+ImagingTable::ImagingTable(
+    const ImagingTable& other,
+    std::function<bool(const ImagingTableEntry&)> isSelected)
+    : _entries(), _independentGroups(), _facets(), _squaredGroups() {
+  std::copy_if(other._entries.begin(), other._entries.end(),
+               std::back_inserter(_entries),
+               [&](const EntryPtr& entry) { return isSelected(*entry); });
   Update();
 }
 
@@ -70,12 +77,14 @@ void ImagingTable::PrintEntry(const ImagingTableEntry& entry) {
 }
 
 void ImagingTable::updateGroups(
-    Groups& groups,
-    std::function<size_t(const ImagingTableEntry&)> getIndex) const {
+    Groups& groups, std::function<size_t(const ImagingTableEntry&)> getIndex,
+    std::function<bool(const ImagingTableEntry&)> isSelected) const {
   std::map<size_t, Group> groupMap;
 
   for (const EntryPtr& e : _entries) {
-    groupMap[getIndex(*e)].push_back(e);
+    if (isSelected(*e)) {
+      groupMap[getIndex(*e)].push_back(e);
+    }
   }
 
   groups.clear();
@@ -99,9 +108,20 @@ void ImagingTable::AssignGridDataFromPolarization(
   }
 }
 
-std::unique_ptr<DeconvolutionTable> ImagingTable::CreateDeconvolutionTable(
+std::unique_ptr<radler::WorkTable> ImagingTable::CreateDeconvolutionTable(
     int n_deconvolution_channels, CachedImageSet& psf_images,
     CachedImageSet& model_images, CachedImageSet& residual_images) const {
+  if (_entries.empty()) {
+    throw std::runtime_error(
+        "Can not create a DeconvolutionTable from an empty ImagingTable.");
+  }
+
+  if (_facets.empty()) {
+    throw std::runtime_error(
+        "Member _facets of ImagingTable is empty. Probably Update() was not "
+        "called after adding entries.");
+  }
+
   // In a DeconvolutionTable the output channel indices range from
   // 0 to (#channels - 1). In an ImagingTable that forms an indepent group,
   // output channel indices may start at a higher index.
@@ -112,11 +132,42 @@ std::unique_ptr<DeconvolutionTable> ImagingTable::CreateDeconvolutionTable(
   const int n_original_channels =
       _entries.back()->outputChannelIndex + 1 - channel_index_offset;
 
-  auto table = std::make_unique<DeconvolutionTable>(
-      n_original_channels, n_deconvolution_channels, channel_index_offset);
+  // Gather dd-psf info from ImagingTable
+  // The ImagingTable is a fairly free format
+  // The following addional assumptions are needed to create a valid
+  // WorkTable with ddp-psfs
+  //
+  // * The dd-psfs are the same, and in the same order for all images
+  // * the facet_ids of the dd-psfs form a simple sequence 0,1,2,...
+  //
+
+  std::vector<radler::PsfOffset> psf_offsets;
+  std::vector<std::shared_ptr<schaapcommon::facets::Facet>> psf_facets;
+
+  // There could be multiple independent groups so the first
+  // facetGroupIndex is not necessarily zero.
+  // Note that looping over _facetGroups.front() is not possible here, since it
+  // does not contain the DD PSF entries.
+  size_t first_facet_group_index = (*_entries.begin())->facetGroupIndex;
+
+  // Although an ImagingTable may contain entries for multiple polarizations,
+  // there should only be DD PSF entries for the first polarization, since the
+  // DD PSF layout is equal for all polarizations.
+  for (const EntryPtr& entry_ptr : _entries) {
+    if (entry_ptr->facetGroupIndex != first_facet_group_index) break;
+    if (!entry_ptr->isDdPsf) continue;
+    const schaapcommon::facets::PixelPosition centre =
+        entry_ptr->facet->GetTrimmedBoundingBox().Centre();
+    psf_offsets.emplace_back(centre.x, centre.y);
+    psf_facets.push_back(entry_ptr->facet);
+  }
+
+  auto table = std::make_unique<radler::WorkTable>(
+      std::move(psf_offsets), n_original_channels, n_deconvolution_channels,
+      channel_index_offset);
   int max_squared_index = -1;
 
-  for (const EntryPtr& entry_ptr : _entries) {
+  for (const EntryPtr& entry_ptr : _facets.front()) {
     assert(entry_ptr);
 
     if (entry_ptr->imageCount >= 1) {
@@ -130,18 +181,18 @@ std::unique_ptr<DeconvolutionTable> ImagingTable::CreateDeconvolutionTable(
         psf_images_ptr = &psf_images;
       }
 
-      std::unique_ptr<DeconvolutionTableEntry> real_entry =
-          entry_ptr->CreateDeconvolutionEntry(channel_index_offset,
-                                              psf_images_ptr, model_images,
-                                              residual_images, false);
+      std::unique_ptr<radler::WorkTableEntry> real_entry =
+          entry_ptr->CreateDeconvolutionEntry(
+              channel_index_offset, psf_images_ptr, model_images,
+              residual_images, psf_facets, false);
       table->AddEntry(std::move(real_entry));
     }
 
     if (entry_ptr->imageCount == 2) {
-      std::unique_ptr<DeconvolutionTableEntry> imaginary_entry =
+      std::unique_ptr<radler::WorkTableEntry> imaginary_entry =
           entry_ptr->CreateDeconvolutionEntry(channel_index_offset, nullptr,
                                               model_images, residual_images,
-                                              true);
+                                              psf_facets, true);
       table->AddEntry(std::move(imaginary_entry));
     }
   }
