@@ -2,12 +2,13 @@
 #define WSCLEAN_IMAGING_TABLE_H
 
 #include "imagingtableentry.h"
-#include "../deconvolution/deconvolutiontable.h"
 
 #include <functional>
 #include <memory>
 #include <vector>
 
+#include <radler/work_table.h>
+
 /**
  * The ImagingTable contains ImagingTableEntry's and supports creating subtables
  * with entries that have the same attribute.
@@ -56,6 +57,11 @@ class ImagingTable {
 
   ImagingTable() = default;
 
+  ImagingTable(const ImagingTable& other,
+               std::function<bool(const ImagingTableEntry&)> isSelected);
+
+  explicit ImagingTable(const Group& entries);
+
   size_t IndependentGroupCount() const { return _independentGroups.size(); }
 
   ImagingTable GetIndependentGroup(size_t index) const {
@@ -70,23 +76,23 @@ class ImagingTable {
 
   const Groups& SquaredGroups() const { return _squaredGroups; }
 
-  size_t FacetGroupCount() const { return _facetGroups.size(); }
-
   // When an imagingtable is split into different tables, a facetGroupIndex may
   // be greater-equal than FacetGroupCount(). Since facetGroupIndices are used
   // for acquiring scheduler locks, always use (MaxFacetGroupIndex() + 1) for
   // determining the number of scheduler locks.
-  size_t MaxFacetGroupIndex() const {
-    // _facetGroups is sorted, since Update() converts a sorted map to it.
-    return _facetGroups.back().front()->facetGroupIndex;
-  }
+  size_t MaxFacetGroupIndex() const { return _entries.back()->facetGroupIndex; }
 
-  ImagingTable GetFacetGroup(size_t index) const {
-    return ImagingTable(_facetGroups[index]);
+  const Groups FacetGroups(bool include_dd_psf) const {
+    Groups facet_groups;
+    updateGroups(
+        facet_groups,
+        [](const ImagingTableEntry& e) { return e.facetGroupIndex; },
+        [include_dd_psf](const ImagingTableEntry& e) {
+          return include_dd_psf || !e.isDdPsf;
+        });
+    return facet_groups;
   }
 
-  const Groups& FacetGroups() const { return _facetGroups; }
-
   size_t FacetCount() const { return _facets.size(); }
 
   ImagingTable GetFacet(size_t index) const {
@@ -121,10 +127,9 @@ class ImagingTable {
   void Update() {
     updateGroups(_independentGroups,
                  [](const ImagingTableEntry& e) { return e.joinedGroupIndex; });
-    updateGroups(_facetGroups,
-                 [](const ImagingTableEntry& e) { return e.facetGroupIndex; });
-    updateGroups(_facets,
-                 [](const ImagingTableEntry& e) { return e.facetIndex; });
+    updateGroups(
+        _facets, [](const ImagingTableEntry& e) { return e.facetIndex; },
+        [](const ImagingTableEntry& e) { return !e.isDdPsf; });
     updateGroups(_squaredGroups, [](const ImagingTableEntry& e) {
       return e.squaredDeconvolutionIndex;
     });
@@ -141,23 +146,25 @@ class ImagingTable {
    */
   void AssignGridDataFromPolarization(aocommon::PolarizationEnum polarization);
 
-  std::unique_ptr<DeconvolutionTable> CreateDeconvolutionTable(
+  std::unique_ptr<radler::WorkTable> CreateDeconvolutionTable(
       int n_deconvolution_channels, CachedImageSet& psf_images,
       CachedImageSet& model_images, CachedImageSet& residual_images) const;
 
  private:
-  explicit ImagingTable(const Group& entries);
-
   static void PrintEntry(const ImagingTableEntry& entry);
   void updateGroups(
-      Groups& groups,
-      std::function<size_t(const ImagingTableEntry&)> getIndex) const;
+      Groups& groups, std::function<size_t(const ImagingTableEntry&)> getIndex,
+      std::function<bool(const ImagingTableEntry&)> isSelected =
+          [](const ImagingTableEntry&) { return true; }) const;
 
   Group _entries;
 
   Groups _independentGroups;
-  Groups _facetGroups;
   Groups _facets;
+  /**
+   * Item i will contain a Group for which each entry has a
+   * squaredDeconvolutionIndex equal to i.
+   */
   Groups _squaredGroups;
 };
 
